/*
 * IndexIndicatorException
 *
 * Author: Lasse Collin <lasse.collin@tukaani.org>
 *
 * This file has been put into the public domain.
 * You can do whatever you want with this file.
 */

package org.tukaani.xz;

class IndexIndicatorException extends Exception {
    private static final long serialVersionUID = 1L;
}
