/*
 * StreamFlags
 *
 * Author: Lasse Collin <lasse.collin@tukaani.org>
 *
 * This file has been put into the public domain.
 * You can do whatever you want with this file.
 */

package org.tukaani.xz.common;

public class StreamFlags {
    public int checkType = -1;
    public long backwardSize = -1;
}
